package edu.gmu.laits;

/**
 * Title:        XML parser
 * Description:  parse XML
 * Copyright:    Copyright (c) 2002
 * Company:      LAITS.GMU
 * @author Peisheng Zhao
 * @version 1.0
 */
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.io.*;
import java.net.*;
import java.util.Vector;
import org.w3c.dom.*;
import edu.gmu.laits.util.WebData;

public class XMLParser implements Serializable {   //reading capability XML file into DOM

  static Document document;
  DocumentBuilder builder;
  Vector nodes;

  /**
   * Constructor       reading local file
   * @param fileName   local file name
   * @throws IOException
   */
  public XMLParser(String fileName) throws IOException {    //
    nodes = new Vector();
    init(new File(fileName));
  }

  /**
   * Constructor   reading file from URL through get method
   * @param url    file URL
   * @throws Exception
   */
  public XMLParser(URL url) throws Exception {   //

    init(WebData.InGet(url));
  }

  /**
   * constructor  reading file from URL through post method
   * @param url         file URL
   * @param para        post string
   * @throws Exception
   */
  public XMLParser(URL url,String para) throws Exception {   //
    init(WebData.InPost(url,para));

  }

  /**
   * constructor           using input stream to initiate parser
   * @param ss             input stream
   * @throws Exception
   */
  public XMLParser(InputStream ss) throws Exception {
    init(ss);
  }

  /**
   * To get Document
   * @return  Document
   */
  public Document getDocument() {return document;}


  void init(Object source) throws IOException{
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    try {
      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new org.xml.sax.ErrorHandler() {
        // ignore fatal errors (an exception is guaranteed)
        public void fatalError(SAXParseException exception) throws SAXException {}
        // treat validation errors as fatal
        public void error(SAXParseException e) throws SAXParseException {throw e;}
        // dump warnings too
        public void warning(SAXParseException err) throws SAXParseException {
          System.out.println("** Warning"+ ", line " + err.getLineNumber()+ ", uri " + err.getSystemId());
          System.out.println(" " + err.getMessage());
        }
      } );
      document=null;
      if(source instanceof File) document = builder.parse((File)source );
      if(source instanceof InputStream) document = builder.parse((InputStream)source );
    }
    catch (SAXException sxe) {
      // Error generated during parsing)
      System.out.println("XMLParser.init SAXException ");
       Exception x = sxe;
      if (sxe.getException() != null)
        x = sxe.getException();
        x.printStackTrace();
        document =null;
    }
    catch (ParserConfigurationException pce) {
      // Parser with specified options can't be built
      pce.printStackTrace();
    }
    catch(IOException ioe) {ioe.printStackTrace();throw ioe;}
  }

  /**
   * To get child count of node
   * @param node   Node
   * @return       child count
   */
  public int childCount(org.w3c.dom.Node node) {
      return node.getChildNodes().getLength();
    }

    /**
     * Leaf or not
     * @param node  Node
     * @return      ture: leaf, flase: not leaf
     */
  public boolean isLeaf(Node node) {
    // Determines whether the icon shows up to the left.
    // Return true for any node with no children
    if (childCount(node) > 1) return false;
    return true;
  }

  private void addNodes(Node bnode,String nodeName) {
    if(bnode==null) return;
    Node node = null;//System.out.println(bnode.getNodeName()+":"+bnode.getgetLocalName());
    if(bnode.getNodeType()==ELEMENT_TYPE && ignoreNameSpace(bnode.getNodeName()).equalsIgnoreCase(nodeName)){
        nodes.add(bnode);//System.out.println("in:"+bnode.toString());
        return;
    }
    for(int i=0;i<bnode.getChildNodes().getLength();i++) {
      addNodes(bnode.getChildNodes().item(i),nodeName);
    }
  }

  /**
   * To get nodes whose names is the specified name
   * @param nodeName    specified name
   * @return            vector containing nodes
   */
  public Vector getNodes(String nodeName) {   //from beginning to get nodes
    getNodes(document,nodeName);
    return nodes;
  }

  /**
   * To get nodes whose name and parent are specified
   * @param parent        parent node
   * @param childName     node name
   * @return              vector containing nodes
   */
  public Vector getNodes(Node parent, String childName) {
    //if(nodes== null)
    nodes = new Vector();
    //nodes.removeAllElements();
    addNodes(parent,childName);
    return nodes;
  }

  /**
   * To get node whose name and parent are specified
   * @param node       parent node
   * @param nodeName   node name
   * @return           Node which is first satisfied the constraints
   */
  public Node getNode(Node node, String nodeName) {
    Node des_Node = null;
    if(node == null) return null;
    //nodeName = ignoreNameSpace(nodeName);
    if(node.getNodeType()==ELEMENT_TYPE && ignoreNameSpace(node.getNodeName()).equalsIgnoreCase(nodeName)){
        des_Node = node;
    }
    for(int i=0;i<node.getChildNodes().getLength();i++) {
      if(des_Node != null) break;
      des_Node = getNode(node.getChildNodes().item(i),nodeName);
    }
    return des_Node;
  }

  /**
   * To get a node whose name is specified
   * @param nodeName    node name
   * @return            Node which is first satisfied the constraints
   */
  public Node getNode(String nodeName) {
    return getNode(document,nodeName);
  }

  /**
   * To get first level child nodes
   * @param parent      parent node
   * @return            vector containing first level child nodes
   */
  public Vector getChild(Node parent) {
     Vector children = new Vector();
     if(parent == null) return children;
     for(int i=0;i<parent.getChildNodes().getLength();i++) {
       if(parent.getChildNodes().item(i).getNodeType()==ELEMENT_TYPE) {
         children.add(parent.getChildNodes().item(i));
       }
     }
     return children;
  }

  /**
   * To get first level child nodes whose name is specified
   * @param parent       parent node
   * @param sub_Name     child node name
   * @return             vector containing child nodes
   */
  public Vector getChild(Node parent,String sub_Name) {
    Vector children = new Vector();
    if( parent  == null)
     return children;
     for(int i=0;i<parent.getChildNodes().getLength();i++) {
       if(parent.getChildNodes().item(i).getNodeType()==ELEMENT_TYPE &&
          ignoreNameSpace(parent.getChildNodes().item(i).getNodeName()).equalsIgnoreCase(sub_Name)) {
         children.add(parent.getChildNodes().item(i));
       }
     }
     return children;
  }

  /**
   * To get the values of first level child nodes whose name is specified
   * @param parent        parent node
   * @param sub_Name      child node name
   * @return              vector contains child node value
   */
  public Vector getChildValue(Node parent,String sub_Name) {
    Vector children = new Vector();
     for(int i=0;i<parent.getChildNodes().getLength();i++) {
       if(parent.getChildNodes().item(i).getNodeType()==ELEMENT_TYPE &&
         ignoreNameSpace(parent.getChildNodes().item(i).getNodeName()).equalsIgnoreCase(sub_Name)) {
         children.add(parent.getChildNodes().item(i).getNodeValue());
       }
     }
     return children;
  }

 /**
  * all sub-elements value within this node are cancatered in a string
  * @param node   parent node
  * @param n      output format, generally is 0 at the beginning
  * @return       cancatered string
  */
  public String getSimpleValue(Node node, int n) {   // n is for .
     if(node==null) return null;
     String s = "";
     if(isLeaf(node)) {//System.out.println("leaf:"+node.getChildNodes().getLength());
       for(int i=0;i<n*2;i++) {s =s+" ";}
       s = s+node.getNodeName()+":";
       if(node.getChildNodes().getLength()==0 || node.getAttributes().getLength()>0){
         String[] ss = getAttributes(node);
         for(int j=0;j<ss.length;j++){
            s = s+ss[j]+" ";
         }
         s = s+"\n";
       }
       else s = s+node.getChildNodes().item(0).getNodeValue()+"\n";
       return s;
     }
     for(int k=0;k<n*2;k++) {s =s+" ";}
     s = s+node.getNodeName()+"\n"; n = n+1;
     for(int i=0;i<node.getChildNodes().getLength();i++) {
       if(!node.getChildNodes().item(i).getNodeName().equalsIgnoreCase("#text"))
        {//System.out.println(node.getChildNodes().item(i).getNodeName());
          s = s+getSimpleValue(node.getChildNodes().item(i),n);}
     }
     return s;
  }

  /**
   * If node is a leaf, then return its value, else return null
   * @param node      node
   * @return          node value
   */
  public String getNodeValue(Node node) {
    try{
      if(isLeaf(node))
        return node.getChildNodes().item(0).getNodeValue();
    }
    catch(Exception e) {return null;}
    return null;
  }

  /**
   * If node is a leaf, then return its value, else return null
   * @param node            node
   * @param ignoreTag       true
   * @return                node value
   */
  public String getNodeValue(Node node,boolean ignoreTag) {
    try{
     // if(ignoreTag)
        return node.getChildNodes().item(0).getNodeValue();
    }
    catch(Exception e) {return null;}
    //return null;
  }

  /**
   * To get specified node value
   * @param nodeName   node name
   * @return           node value
   */
  public String getValue(String nodeName) {
    Node node = getNode(nodeName);
    if(node==null) return null;
    return getNodeValue(node);
  }


  /**
   * To get specfied child node value
   * @param node      parent node
   * @param nodeName  child node name
   * @return          child node value
   */
  public String getValue(Node node, String nodeName){
    Node des_Node = getNode(node,nodeName);
    if(des_Node==null) return null;
    String s=getNodeValue(des_Node,true);
    return s;
  }

  /**
   * To get specfied child node value
   * @param node       parent node
   * @param nodeName   child node name
   * @param ignoreTag  true
   * @return           node value
   */
  public String getValue(Node node, String nodeName,boolean ignoreTag){
    Node des_Node = getNode(node,nodeName);
    if(des_Node==null) return null;
    String s=getNodeValue(des_Node,ignoreTag);
    return s;
  }

  /**
   * To get the values of children whose parent is specified
   * @param node            parent node
   * @param nodeName        children names
   * @return                vector contains the values of children
   */
  public Vector getValues(Node node,String nodeName) {
    Vector values = new Vector();
    Vector nodes = getNodes(node,nodeName);
    for(int i=0;i<nodes.size();i++) {
      values.add(getNodeValue((Node)nodes.get(i)));
    }
    return values;
  }

  /**
   * To get values of specified nodes
   * @param nodeName  node name
   * @return          vector containing values of nodes
   */
  public Vector getValues(String nodeName) {
     return getValues(document,nodeName);
   }

   /**
    * To get the values of node attributes
    * @param node        node
    * @return            values of attributes
    */
  public String[] getAttributes(Node node) {
    if(node == null) return null;
    String attributes[] = new String[node.getAttributes().getLength()];
    for(int i=0;i<node.getAttributes().getLength();i++) {
      attributes[i] = " "+node.getAttributes().item(i).getNodeName()+"="+node.getAttributes().item(i).getNodeValue();
    }
    return attributes;
  }

  /**
   * To get specified attribute value
   * @param node            node
   * @param attName         attribute name
   * @return                attribute value
   */
  public String getAttribute(Node node, String attName) {
    if(node == null) return null;
    for(int i=0;i<node.getAttributes().getLength();i++) {
      if(node.getAttributes().item(i).getNodeName().equalsIgnoreCase(attName)) {
        return node.getAttributes().item(i).getNodeValue();
      }
    }
    return null;
  }

  /**
   * Ignorre name space
   * @param s   the name with name space
   * @return    the name without name space
   */
  public String ignoreNameSpace(String s) {
    int index = s.indexOf(":");
    if(index >= 0) {
      s = s.substring(index+1);
    }
    return s;
  }
  static final int ELEMENT_TYPE = 1;
  static final int ATTR_TYPE    = 2;
  static final int TEXT_TYPE    = 3;
  static final int CDATA_TYPE   = 4;
  static final int ENTITYREF_TYPE = 5;
  static final int ENTITY_TYPE    = 6;
  static final int PROCINSTR_TYPE = 7;
  static final int COMMENT_TYPE   = 8;
  static final int DOCUMENT_TYPE  = 9;
  static final int DOCTYPE_TYPE   = 10;
  static final int DOCFRAG_TYPE   = 11;
  static final int NOTATION_TYPE  = 12;

}
