package edu.gmu.laits.registry;

/**
 * Title:        Registry Parser
 * Description:  It can get map image via parsing capability to get right information.
 * Copyright:    Copyright (c) 2002
 * Company:      LAITS.GMU
 * @author       Peisheng Zhao
 * @version 1.0
 */
import java.io.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import laits.registry.ogc.infomodel.*;
import laits.registry.ogc.infomodel.core.*;
import edu.gmu.laits.XMLParser;


public class RegistryParser {


  public static String matched;
  public static String next;
  /**
   * Parse the result of OGC service query
   * @param url     CSW URL
   * @param key     OGC service query
   * @return        Service list
   */
  public static ArrayList parseServiceResponse(URL url, String key) {
    ArrayList alist = new ArrayList();
    try {
      XMLParser parser = new XMLParser(url,key);
      int number = Integer.parseInt(parser.getAttribute(parser.getNode("SearchResults"),"numberOfRecordsReturned"));
      if(number == 0) return alist;
      Vector vec = parser.getNodes("Service");
      for(int i=0;i<vec.size();i++) {
        Node node = (Node)vec.get(i);
        Service service = new Service();
        service.setTypeString("Service");
        setDetail(service,node);
        service.setID(parser.getAttribute(node,"id"));
        service.setObjectType(parser.getAttribute(node,"objectType"));
        Node nameNode = parser.getNode(node,"Name");
        service.setName(parser.getAttribute(parser.getNode(nameNode,"LocalizedString"),"value"));
        Node desNode = parser.getNode(node,"Description");
        service.setDescription(parser.getAttribute(parser.getNode(desNode,"LocalizedString"),"value"));
        Vector slotNodes = parser.getNodes(node,"Slot");
        for(int j=0;j<slotNodes.size();j++) {
          Node nd = (Node)slotNodes.get(j);
          String slotName = parser.getAttribute(nd,"name");
          if(slotName.equalsIgnoreCase("serviceType"))
            service.setSlotValue("serviceType",parser.getValue(nd,"Value"));
          else if(slotName.equalsIgnoreCase("connectPointLinkage"))
            service.setSlotValue("connectPointLinkage",parser.getValue(nd,"Value"));
        }
        alist.add(service);
      }
    }
    catch(Exception e) {e.printStackTrace();}
    return alist;
  }



  /**
 * Parse the result of granule query
 * @param url     CSW URL
 * @param key     granule query
 * @return        ExtrinsicObject(Dataset) list
 */

 public static ArrayList parseGranuleResponse(URL url, String key) {
    ArrayList alist = new ArrayList();
   try {
     XMLParser parser = new XMLParser(url,key);
     matched = parser.getAttribute(parser.getNode("SearchResults"),"numberOfRecordsMatched");

     if(matched.equals("0")) return alist;
     next = parser.getAttribute(parser.getNode("SearchResults"),"nextRecord");
     Vector vec = parser.getNodes("DataGranule");
     if(vec.size()==0) vec = parser.getNodes("OGCExtrinsicObject") ;
     for(int i=0;i<vec.size();i++) {
       Node node = (Node)vec.get(i);
       DataGranuleObject eo = new DataGranuleObject();
       eo.setTypeString("DataGranule");
       setDetail(eo,node);
       eo.setID(parser.getAttribute(node,"id"));
       eo.setObjectType(parser.getAttribute(node,"objectType"));
       Node nameNode = parser.getNode(node,"Name");
      // if(nameNode == null) nameNode = parser.getNode(node,"ns1:Name");
      // eo.setName(parser.getAttribute(parser.getNode(nameNode,"LocalizedString"),"value"));
       eo.setName(parser.getNodeValue(nameNode));
       Node desNode = parser.getNode(node,"Description");
       //if(desNode==null) desNode = parser.getNode(node,"ns2:Description");
       //eo.setDescription(parser.getAttribute(parser.getNode(desNode,"LocalizedString"),"value"));
       eo.setDescription(parser.getNodeValue(desNode));
       Node purlNode = parser.getNode("previewURL");
       if(purlNode != null)
         eo.setPreviewURL(parser.getNodeValue(purlNode));
       Node sizeNode = parser.getNode("sizeMBDataGranule");
       if(sizeNode != null)
         eo.setDataSize(parser.getNodeValue(sizeNode));
       Node formatNode = parser.getNode("dataFormat");
       if(formatNode != null)
         eo.setFormat(parser.getNodeValue(formatNode));
       Vector bbNodes = parser.getNodes("BBOX");
       String bbox = "";
       for(int j=0;j<bbNodes.size();j++) {
         Node bNode = (Node)bbNodes.get(j);
         bbox += parser.getValue(bNode,"referenceSystemNameCode");
         bbox += "  ";
         bbox += parser.getValue(bNode, "westBoundingCoordinate");
         bbox += ",";
         bbox += parser.getValue(bNode, "southBoundingCoordinate");
         bbox += " ";
         bbox += parser.getValue(bNode, "eastBoundingCoordinate");
         bbox += ",";
         bbox += parser.getValue(bNode, "northBoundingCoordinate");
         if(j!= bbNodes.size()-1)
         bbox += ";\n";
       }
       eo.setBBox(bbox);
       Vector urlNodes = parser.getNodes(node, "OnlineAccessURLs");
       //Vector urls = parser.getNodes(urlNode, "URL");System.out.println(urls.size());
       for(int j=0;j<urlNodes.size();j++) {
         Node uNode = (Node)urlNodes.get(j);//System.out.println(parser.getValue(uNode,"MimeType"));
         //System.out.println(parser.getNodeValue(uNode));
         String mType = parser.getValue(uNode,"MimeType");
         String tURL = parser.getValue(uNode,"URL");
         if(mType != null && tURL != null)
         eo.addOnlineResource(mType,tURL);

       }

       alist.add(eo);
     }
   }
   catch(Exception e) {e.printStackTrace();}
   return alist;
 }





  /**
  * Parse the result of data query
  * @param url     CSW URL
  * @param key     data query
  * @return        ExtrinsicObject(Dataset) list
  */

  public static ArrayList parseDataResponse(URL url, String key) {
     ArrayList alist = new ArrayList();
    try {
      XMLParser parser = new XMLParser(url,key);
      matched = parser.getAttribute(parser.getNode("SearchResults"),"numberOfRecordsMatched");
      if(matched.equals("0")) return alist;
      next = parser.getAttribute(parser.getNode("SearchResults"),"nextRecord");
      Vector vec = parser.getNodes("Dataset");
      if(vec.size()==0) vec = parser.getNodes("OGCExtrinsicObject") ;
      for(int i=0;i<vec.size();i++) {
        Node node = (Node)vec.get(i);
        ExtrinsicObject eo = new ExtrinsicObject();
        eo.setTypeString("Dataset");
        setDetail(eo,node);
        eo.setID(parser.getAttribute(node,"id"));
        eo.setObjectType(parser.getAttribute(node,"objectType"));
        Node nameNode = parser.getNode(node,"Name");
       // if(nameNode == null) nameNode = parser.getNode(node,"ns1:Name");
        //eo.setName(parser.getAttribute(parser.getNode(nameNode,"LocalizedString"),"value"));
         eo.setName(parser.getNodeValue(nameNode));
        Node desNode = parser.getNode(node,"Description");
        //if(desNode==null) desNode = parser.getNode(node,"ns2:Description");
        //eo.setDescription(parser.getAttribute(parser.getNode(desNode,"LocalizedString"),"value"));
        eo.setDescription(parser.getNodeValue(desNode));
        alist.add(eo);
      }
    }
    catch(Exception e) {}
    return alist;
  }

  /**
  * Parse the result of data query
  * @param url     CSW URL
  * @param key     data query
  * @return        ExtrinsicObject(Dataset) list
  */

  public static ArrayList parseCoverageResponse(URL url, String key) {
     ArrayList alist = new ArrayList();
    try {System.out.println("coverage:"+key);
      XMLParser parser = new XMLParser(url,key);
      matched = parser.getAttribute(parser.getNode("SearchResults"),"numberOfRecordsMatched");
      System.out.println(matched);
      if(matched.equals("0")) return alist;
      next = parser.getAttribute(parser.getNode("SearchResults"),"nextRecord");
      Vector vec = parser.getNodes("WCSCoverage");
      if(vec.size()==0) vec = parser.getNodes("OGCExtrinsicObject") ;
      for(int i=0;i<vec.size();i++) {
        Node node = (Node)vec.get(i);
        WCSCoverage eo = new WCSCoverage();
        eo.setTypeString("Dataset");
        setDetail(eo,node);
        eo.setID(parser.getAttribute(node,"id"));
        eo.setObjectType(parser.getAttribute(node,"objectType"));
        Node nameNode = parser.getNode(node,"Name");
       // if(nameNode == null) nameNode = parser.getNode(node,"ns1:Name");
        //eo.setName(parser.getAttribute(parser.getNode(nameNode,"LocalizedString"),"value"));
        eo.setName(parser.getNodeValue(nameNode));
        Node desNode = parser.getNode(node,"Description");
        //if(desNode==null) desNode = parser.getNode(node,"ns2:Description");
        eo.setDescription(parser.getAttribute(parser.getNode(desNode,"LocalizedString"),"value"));
        eo.setDescription(parser.getNodeValue(desNode));
        eo.setGranule(parser.getValue(node,"granule"));
        eo.setRequestCRSs(parser.getValues(node,"requestCRSs"));
        eo.setResponseCRSs(parser.getValues(node,"responseCRSs"));
        eo.setSupportFormats(parser.getValues(node,"supportFormats"));
        alist.add(eo);
      }
    }
    catch(Exception e) {}
    return alist;
  }




  /**
  * Parse the result of service-data query
  * @param url     CSW URL
  * @param key     Service-data query
  * @return        Service list
  */

  public static ArrayList parseDataServiceResponse(URL url, String key) {
     ArrayList alist = new ArrayList();
     try {
       XMLParser parser = new XMLParser(url,key);
       int number = Integer.parseInt(parser.getAttribute(parser.getNode("searchResults"),"numberOfRecordsReturned"));
       //System.out.println("number:"+number);
       if(number == 0) return alist;
       Vector vec = parser.getNodes("Service");
       for(int i=0;i<vec.size();i++) {
         Node node = (Node)vec.get(i);
         Service service = new Service();
         setDetail(service,node);
         service.setID(parser.getAttribute(node,"id"));
         //service.setObjectType(parser.getAttribute(node,"objectType"));
         Vector slots = parser.getNodes(node,"Slot");
         for(int k=0;k<slots.size();k++) {
           Node slot = (Node)slots.get(k);
           String slotName = parser.getAttribute(slot,"name");
           if(slotName.equalsIgnoreCase("serviceType")) {
             service.setType("OGCService",parser.getValue(slot,"Value"));

           }
           else if(slotName.equalsIgnoreCase("version")) {
             service.setVersion(parser.getValue(slot,"Value"));

           }
           else if(slotName.equalsIgnoreCase("connectPointLinkage"))
            service.setSlotValue("connectPointLinkage",parser.getValue(slot,"Value"));

         }
         service.setTypeString("Service");
         Node nameNode = parser.getNode(node,"Name");
         //service.setName(parser.getAttribute(parser.getNode(nameNode,"LocalizedString"),"value"));
         service.setName(parser.getNodeValue(nameNode));
         Node desNode = parser.getNode(node,"Description");
         //service.setDescription(parser.getAttribute(parser.getNode(desNode,"LocalizedString"),"value"));
         service.setDescription(parser.getNodeValue(desNode));
         Vector bindings = parser.getNodes(node,"ServiceBinding");
         for(int j=0;j<bindings.size();j++) {
           Node bnode = (Node)bindings.get(j);
           ServiceBinding sb = new ServiceBinding();
           sb.setID(parser.getAttribute(bnode,"id"));
           sb.setObjectType(parser.getAttribute(bnode,"objectType"));
           sb.setAccessURI(parser.getAttribute(bnode,"accessURI"));
           Node bnameNode = parser.getNode(bnode,"Name"); //System.out.println("bindname"+parser.getAttribute(parser.getNode(bnameNode,"LocalizedString"),"value"));
           //sb.setName(parser.getAttribute(parser.getNode(bnameNode,"LocalizedString"),"value"));
           sb.setName(parser.getNodeValue(bnameNode));
           service.getServiceBindings().add(sb);
         }
         alist.add(service);
       }
     }
     catch(Exception e) {}
     return alist;
  }

  /**
   * Detail is raw XML document for the nod and registryobject
   * @param ro      registry Object
   * @param node    the source node
   */
  private static void setDetail(RegistryObject ro, Node node){
    //
    try {
    StringWriter sw = new StringWriter();
    TransformerFactory tFactory = TransformerFactory.newInstance();
    Transformer transformer = tFactory.newTransformer();
    DOMSource source = new DOMSource(node);
    StreamResult result = new StreamResult(sw);
    transformer.transform(source, result);
    ro.setDetail(sw.toString());
    }catch(Exception e) {e.printStackTrace();}
  }

}