package edu.gmu.laits.registry;

/**
 * Title:        Request Factory
 * Description:  Create CSW request.
 * Copyright:    Copyright (c) 2002
 * Company:      LAITS.GMU
 * @author       Peisheng Zhao
 * @version 1.0
 */
import java.util.*;
//import laits.registry.ogc.infomodel.core.HDFObject;
//import edu.gmu.laits.hdf.HDFObjectDescription;

public class RequestFactory {


  /** request header */
  public static String header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><csw:GetRecords xmlns=\"http://www.opengis.net/cat/csw\""+
     " xmlns:csw=\"http://www.opengis.net/cat/csw\" xmlns:ogc=\"http://www.opengis.net/ogc\" xmlns:gml=\"http://www.opengis.net/gml\" version=\"1.0\""+
    " responseHandler=\"\" outputFormat=\"text/xml\" charset=\"UTF-8\" outputSchema=\"http://www.opengis.net/cat/csw\""+
   " startPosition=\"1\" maxRecords=\"15\">"+
   " <csw:distributedSearch propagate=\"ifNoLocal\" hopCount=\"2\" collectionId=\"\"/>";
  public static void setPosition (String p) {
    header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><csw:GetRecords xmlns=\"http://www.opengis.net/cat/csw\""+
     " xmlns:csw=\"http://www.opengis.net/cat/csw\" xmlns:ogc=\"http://www.opengis.net/ogc\" xmlns:gml=\"http://www.opengis.net/gml\" version=\"1.0\""+
    " responseHandler=\"\" outputFormat=\"text/xml\" charset=\"UTF-8\" outputSchema=\"http://www.opengis.net/cat/csw\""+
   " startPosition=\""+p+"\" maxRecords=\"8\">"+
   " <csw:distributedSearch propagate=\"ifNoLocal\" hopCount=\"2\" collectionId=\"\"/>";

  }


   /**
    * Create request to search OGC service
    * @param sName  service type name, including Web Coverage Service, Web Feature Service,
    *               Web Map Service, Catalog Service for Web
    * @return       query result (in XML)
    */
   public static String createRequest(String sName) {   // seearch service

    String key = "";
    if(sName.equalsIgnoreCase("Web Coverage Service")) key = "WCS"; //"OWS-2200";
    else if(sName.equalsIgnoreCase("Web Feature Service")) key = "WFS";//"OWS-2100";
    else if(sName.equalsIgnoreCase("Web Map Service")) key = "WMS";//"OWS-2300";
    else if(sName.equalsIgnoreCase("Catalog Service for Web")) key = "CSW";//"OWS-2500";

     String r = "<csw:Query typeNames=\"Service ClassificationNode Classification\">"+
                    "<csw:ElementName>/Service/</csw:ElementName>"+
                    "<csw:Constraint version=\"3.0.1\"><ogc:Filter><ogc:And><ogc:PropertyIsEqualTo>"+
                    "<ogc:PropertyName>/Service/@id</ogc:PropertyName>"+
                    "<ogc:PropertyName>/Classification/@classifiedObject</ogc:PropertyName>"+
                     "</ogc:PropertyIsEqualTo>"+
                    "<ogc:PropertyIsEqualTo>"+
                     "<ogc:PropertyName>/Classification/@classificationNode</ogc:PropertyName>"+
                     "<ogc:PropertyName>/ClassificationNode/@id</ogc:PropertyName>"+
                     "</ogc:PropertyIsEqualTo>"+
                     "<ogc:PropertyIsEqualTo>"+
                      "<ogc:PropertyName>/ClassificationNode/@code</ogc:PropertyName>"+
                      "<ogc:Literal>"+key+"</ogc:Literal>"+
                      "</ogc:PropertyIsEqualTo>"+
                      "</ogc:And></ogc:Filter></csw:Constraint></csw:Query></csw:GetRecords>";

      return r;

  }

  /**
   * Create request to search data based on data attribute
   * @param bbox          bouding box, bbox[0] is the name of spatial reference system
   *                      bbox[1] is minX, bbox[2] is minY, bbox[3] is maxX and bbox[4] is maxY
   * @param pName         product name
   * @param iName         instrument short name
   * @param plName        platform short name
   * @param sKeyword      ECS sensor keyword
   * @param tKeyword      ECS topic keyword
   * @param dKeyword      ECS discipline keyword
   * @param termKeyword   ECS term keyword
   * @param vKeyword      ECS variable keyword
   * @param pLevel        processing level
   * @return              query result (in XML)
   */
  /*public static String createRequest(String bbox[], String t[], String pName, String iName, String plName,
                                   String sKeyword,  String tKeyword, String dKeyword, String termKeyword,
                                   String vKeyword, String pLevel) {

     String r = "\n<csw:Query typeNames=\"Dataset\"> \n<csw:ElementSetName>full</csw:ElementSetName>" +
              "<csw:ElementName>/Dataset/</csw:ElementName>" +
              "\n   <csw:Constraint version=\"1.0.0\">" +
              "\n        <ogc:Filter>\n" +
              "             <ogc:And>\n";
    if(iName != "" && iName != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/instrumentShortName</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + iName + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(pName != "" && pName != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/productName</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + pName + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";

    if(plName != "" && plName != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/platformShortName</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + plName + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";

    if(sKeyword != "" && sKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/ECSSensorKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + sKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(tKeyword != "" && tKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/ECSTopicKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + tKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(dKeyword != "" && dKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/ECSDisciplineKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + dKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(termKeyword != "" && termKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/ECSTermKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + termKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(vKeyword != "" && vKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/ECSVariableKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + vKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(pLevel != "" && pLevel != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/Dataset/processingLevel</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + pLevel + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(t != null)
      r = r +
          "                 <ogc:Not>\n" +
          "                   <ogc:PropertyIsLessThan>\n" +
          "                     <ogc:PropertyName>/Dataset/beginDateTime</ogc:PropertyName>\n" +
          "                        <ogc:Literal>" + t[0] + "</ogc:Literal>\n" +
          "                   </ogc:PropertyIsLessThan>\n" +
          "                 </ogc:Not>\n" +
          "                 <ogc:Not>\n" +
          "                   <ogc:PropertyIsGreaterThan>\n" +
          "                     <ogc:PropertyName>/Dataset/endDateTime</ogc:PropertyName>\n" +
          "                        <ogc:Literal>" + t[1] + "</ogc:Literal>\n" +
          "                   </ogc:PropertyIsGreaterThan>\n" +
          "                 </ogc:Not>\n";

    if(bbox != null)
      r = r +
          "                  <ogc:BBOX>\n" +
          "                       <ogc:PropertyName>/Dataset/BBOX</ogc:PropertyName>\n"+
          "                          <gml:Box srsName=\"" + bbox[0] + "\">\n" +
          "                              <gml:coordinates>" + bbox[1] + "," + bbox[2] + " " + bbox[3] + "," + bbox[4] + "</gml:coordinates>\n" +
          "                         </gml:Box>\n" +
          "                  </ogc:BBOX>\n";
       r = r +   "               </ogc:And>\n" +
          "            </ogc:Filter>\n" +
          "         </csw:Constraint>\n" +
          "      </csw:Query>\n" +
          "   </csw:GetRecords>"; System.out.println(r);

    return r;
  }*/



  /**
   * Create request to search data granule based on data attribute
   * @param bbox          bouding box, bbox[0] is the name of spatial reference system
   *                      bbox[1] is minX, bbox[2] is minY, bbox[3] is maxX and bbox[4] is maxY
   * @param pName         product name
   * @param iName         instrument short name
   * @param plName        platform short name
   * @param sKeyword      ECS sensor keyword
   * @param tKeyword      ECS topic keyword
   * @param dKeyword      ECS discipline keyword
   * @param termKeyword   ECS term keyword
   * @param vKeyword      ECS variable keyword
   * @param pLevel        processing level
   * @return              request (in XML)
   */
  public static String createRequest(String[] list, String bbox[], String t[], String pName, String iName, String plName,
                                   String keyword, String sKeyword,  String tKeyword, String dKeyword, String termKeyword,
                                   String vKeyword, String pKeyword, String pLevel) {

     String r = null;

     if(list != null) {
       r = r + "\n<csw:FederationSearch CatalogServiceIDs=\"";
       for(int i=0;i<list.length;i++) {
         r = r + list[i];
         if(i != list.length-1)
           r += ",";
       }
       r += "\"/>";

     }


     r = r +"\n<csw:Query typeNames=\"DataGranule\"> \n<csw:ElementSetName>full</csw:ElementSetName>" +
              "<csw:ElementName>/DataGranule/</csw:ElementName>" +
              "\n   <csw:Constraint version=\"1.0.0\">" +
              "\n        <ogc:Filter>\n" +
              "             <ogc:And>\n";
    if(iName != "" && iName != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/instrumentShortName</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + iName + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(pName != "" && pName != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/productName</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + pName + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";

    if(plName != "" && plName != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/platformShortName</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + plName + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";


    if(sKeyword != "" && sKeyword != null)
          r = r +
              "                 <ogc:PropertyIsEqualTo>\n" +
              "                     <ogc:PropertyName>/DataGranule/SensorShortName</ogc:PropertyName>\n" +
              "                     <ogc:Literal>" + sKeyword + "</ogc:Literal>\n" +
              "                 </ogc:PropertyIsEqualTo>\n";


    if(tKeyword != "" && tKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/TopicKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + tKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(dKeyword != "" && dKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/DisciplineKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + dKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(termKeyword != "" && termKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/TermKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + termKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";
    if(vKeyword != "" && vKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/VariableKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + vKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";

    if(pKeyword != "" && pKeyword != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/parameterKeyword</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + pKeyword + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";

    if(pLevel != "" && pLevel != null)
      r = r +
          "                 <ogc:PropertyIsEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/processingLevel</ogc:PropertyName>\n" +
          "                     <ogc:Literal>" + pLevel + "</ogc:Literal>\n" +
          "                 </ogc:PropertyIsEqualTo>\n";


    if(keyword != "" && keyword != null)
       r = r +
           "                 <ogc:PropertyIsEqualTo>\n" +
           "                     <ogc:PropertyName>/DataGranule/Slot/ValueList/Value</ogc:PropertyName>\n" +
           "                     <ogc:Literal>" + keyword + "</ogc:Literal>\n" +
           "                 </ogc:PropertyIsEqualTo>\n" +
           "                 <ogc:PropertyIsEqualTo>\n" +
           "                     <ogc:PropertyName>/DataGranule/Slot/@name</ogc:PropertyName>\n" +
           "                     <ogc:Literal>" + "keywords" + "</ogc:Literal>\n" +
           "                 </ogc:PropertyIsEqualTo>\n";


    if(t != null)
      r = r +
          "                 <ogc:PropertyIsGreaterThanOrEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/beginDateTime</ogc:PropertyName>\n" +
          "                        <ogc:Literal>" + t[0] + "</ogc:Literal>\n" +
          "                   </ogc:PropertyIsGreaterThanOrEqualTo>\n" +
          "                   <ogc:PropertyIsLessThanOrEqualTo>\n" +
          "                     <ogc:PropertyName>/DataGranule/endDateTime</ogc:PropertyName>\n" +
          "                        <ogc:Literal>" + t[1] + "</ogc:Literal>\n" +
          "                   </ogc:PropertyIsLessThanOrEqualTo>\n";


    if(bbox != null)
      r = r +
          "                  <ogc:BBOX>\n" +
          "                       <ogc:PropertyName>/DataGranule/BBOX</ogc:PropertyName>\n"+
          "                          <gml:Box srsName=\"" + bbox[0] + "\">\n" +
          "                              <gml:coordinates>" + bbox[1] + "," + bbox[2] + " " + bbox[3] + "," + bbox[4] + "</gml:coordinates>\n" +
          "                         </gml:Box>\n" +
          "                  </ogc:BBOX>\n";
       r = r +   "               </ogc:And>\n" +
          "            </ogc:Filter>\n" +
          "         </csw:Constraint>\n" +
          "      </csw:Query>\n" +
          "   </csw:GetRecords>\n\n"; System.out.println(r);

    return r;
  }

  /**
   * Create a request to search the coverage within a granule
   * @param id         coverage ID
   * @param onlineURL  coverage online URL
   * @return           request
   */
  public static String createDataRequest (String id, String onlineURL) {
    String r = "\n<csw:Query typeNames=\"WCSCoverage\"> \n<csw:ElementSetName>full</csw:ElementSetName>" +
              "<csw:ElementName>/WCSCoverage/</csw:ElementName>" +
              "\n   <csw:Constraint version=\"1.0.0\">" +
              "\n        <ogc:Filter>\n" +
              "             <ogc:And>\n";

    r = r +   "                 <ogc:PropertyIsEqualTo>\n" +
              "                     <ogc:PropertyName>/WCSCoverage/granule</ogc:PropertyName>\n" +
              "                     <ogc:Literal>" + id + "</ogc:Literal>\n" +
              "                 </ogc:PropertyIsEqualTo>\n";

    if(onlineURL != null)
    r = r +   "                 <ogc:PropertyIsEqualTo>\n" +
              "                     <ogc:PropertyName>/DataGranule/OnlineAccessURLs/URL</ogc:PropertyName>\n" +
              "                     <ogc:Literal>" + onlineURL + "</ogc:Literal>\n" +
              "                 </ogc:PropertyIsEqualTo>\n";

    r = r +   "               </ogc:And>\n" +
              "            </ogc:Filter>\n" +
              "         </csw:Constraint>\n" +
              "      </csw:Query>\n" +
              "   </csw:GetRecords>\n\n"; System.out.println(r);
   return r;

  }



  /**
   * create reqeust to search service based on associated data
   * @param id    data UUID
   * @return      query result (in XML)
   */
  public static String CreateDataServiceRequest(String id) {
    String header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><csw:GetRecords xmlns=\"http://www.opengis.net/cat/csw\""+
         " xmlns:csw=\"http://www.opengis.net/cat/csw\" xmlns:ogc=\"http://www.opengis.net/ogc\" xmlns:gml=\"http://www.opengis.net/gml\" version=\"1.0\""+
        " responseHandler=\"\" outputFormat=\"text/xml\" charset=\"UTF-8\" outputSchema=\"http://www.opengis.net/cat/csw\""+
       " startPosition=\"1\" maxRecords=\"50\">"+
       " <csw:distributedSearch propagate=\"ifNoLocal\" hopCount=\"2\" collectionId=\"\"/>";

    String r = header +
               "<csw:Query typeNames=\"Service\">\n" +
               "<csw:ElementSetName>full</csw:ElementSetName>\n" +
               "<csw:ElementName>/Service/</csw:ElementName>\n" +
               "<csw:Constraint version=\"1.0.0\">\n" +
               "<ogc:Filter>\n" +
	       "<ogc:And>\n" +
              /* "<ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyName>/Dataset/@id</ogc:PropertyName>\n" +
               "<ogc:Literal>" + id + "</ogc:Literal>\n" +
               "</ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyName>/Dataset/@id</ogc:PropertyName>\n" +
               "<ogc:PropertyName>/Association/@targetObject</ogc:PropertyName>\n" +
               "</ogc:PropertyIsEqualTo>\n" +*/
               "<ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyName>/Association/@targetObject</ogc:PropertyName>\n" +
               "<ogc:Literal>" + id + "</ogc:Literal>\n" +
               "</ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyName>/Service/@id</ogc:PropertyName>\n"+
               "<ogc:PropertyName>/Association/@sourceObject</ogc:PropertyName>\n" +
               "</ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyName>/Association/@associationType</ogc:PropertyName>\n" +
               "<ogc:PropertyName>/ClassificationNode/@id</ogc:PropertyName>\n" +
               "</ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyIsEqualTo>\n" +
               "<ogc:PropertyName>/ClassificationNode/@code</ogc:PropertyName>\n" +
               "<ogc:Literal>OperatesOn</ogc:Literal>\n" +
               "</ogc:PropertyIsEqualTo>\n" +
               "</ogc:And></ogc:Filter></csw:Constraint></csw:Query></csw:GetRecords>";


     return r;

  }

  /**
  * Create a request to search the coverage information
  * @param id         coverage name

  * @return           request
  */
 public static String createCoverageInformaitonRequest (String name) {
   String r = "\n<csw:Query typeNames=\"WCSCoverage\"> \n<csw:ElementSetName>full</csw:ElementSetName>" +
             "<csw:ElementName>/WCSCoverage/</csw:ElementName>" +
             "\n   <csw:Constraint version=\"1.0.0\">" +
             "\n        <ogc:Filter>\n" +
             "             <ogc:And>\n";

   r = r +   "                 <ogc:PropertyIsEqualTo>\n" +
             "                     <ogc:PropertyName>/WCSCoverage/name</ogc:PropertyName>\n" +
             "                     <ogc:Literal>" + name + "</ogc:Literal>\n" +
             "                 </ogc:PropertyIsEqualTo>\n";

  r = r +   "               </ogc:And>\n" +
             "            </ogc:Filter>\n" +
             "         </csw:Constraint>\n" +
             "      </csw:Query>\n" +
             "   </csw:GetRecords>\n\n"; System.out.println(r);
  return r;

 }


}