package laits.registry.ogc.infomodel.core;

/**
 * Title:        HDF Object
 * Description:  Represents registried HDF data.
 * Copyright:    Copyright (c) 2002
 * Company:      LAITS.GMU
 * @author       Peisheng Zhao
 * @version 1.0
 */
import java.net.*;
import java.util.*;
import org.w3c.dom.Node;
import edu.gmu.laits.XMLParser;
import edu.gmu.laits.gis.GM_Envelope;

public class HDFObject extends OGCExtrinsicObject{

  public String productName;
  public String HDFFileName;
  public String abstractt;
  public String productionTime;
  public Hashtable ranges;

  String fileURI;    // data location
  GM_Envelope gme;

  public HDFObject() {
    m_sObjectType = "HDFObject";

  }

  public HDFObject(String contentLocation) {
    this();
    this.m_sContentURL = contentLocation;
    init();
  }

  void init() {
    try {
    XMLParser parser = new XMLParser(new URL(m_sContentURL)); // using getResourceByID to get XML document
    Node hdfNode = parser.getNode("HDFObject");
    this.m_sID = parser.getAttribute(hdfNode,"id");
    productName = parser.getValue(hdfNode,"ProductName");
    HDFFileName = parser.getValue(hdfNode,"HDFFileName");
    abstractt = parser.getValue(hdfNode,"Abstract");
    productionTime = parser.getValue(hdfNode,"ProductionTime");
     Node replicaNode = parser.getNode(hdfNode,"Replica");
    fileURI = parser.getValue(replicaNode,"URL");
    String rangeString = parser.getValue(hdfNode,"Ranges");
    if(rangeString != null) {
       ranges = new Hashtable();
       StringTokenizer st = new StringTokenizer(rangeString,";");
       while (st.hasMoreTokens()) {
           String b = st.nextToken();System.out.println(b);
           int index = b.indexOf(":");
           if(index <0) {
             ranges.put(b," ");
           }
           else {
             StringTokenizer st1 = new StringTokenizer(b,",");
             while(st1.hasMoreTokens()) {
               String[] axis = new String[3];
               String one = st1.nextToken();
               axis[0] = one.substring(index+1);
               axis[1] = st1.nextToken();
               axis[2] = st1.nextToken();
               ranges.put(one.substring(0,index),axis);
             }
           }
        }
    }


    // get spatial information
    //Node metaNode = parser.getNode(hdfNode,"MD_Metadata");
    //Node collectionMetaNode = parser.getNode(metaNode,"COLLECTIONMETADATA");
    //Node spatialMetaNode = parser.getNode(collectionMetaNode,"Spatial");
    Node boundingNode = parser .getNode(hdfNode,"BBox");
    double south = Double.parseDouble(parser.getValue(boundingNode,"SouthBound"));
    double west = Double.parseDouble(parser.getValue(boundingNode,"WestBound"));
    double north = Double.parseDouble(parser.getValue(boundingNode,"NorthBound"));
    double east = Double.parseDouble(parser.getValue(boundingNode,"EastBound"));
    gme = new GM_Envelope(west,south,east-west,north-south);
    }
    catch(Exception e) {e.printStackTrace();}
  }

  public String getURI() {return fileURI;}
  public GM_Envelope getBounding () {return gme;}
}